﻿// MinCMS noninvasive ASP.NET WebForms System
// ----------------------------------------------------
// License: Microsoft Reciprocal License (Ms-RL)
// Version: 1.0.0.0
// Initial Development: Prosodev - Software Company / August 2013 / V 1.0.0.0

namespace MinCms.Core.Ui
{
    using System;
    using System.ComponentModel;
    using System.Globalization;
    using System.Web.UI;
    using System.Web.UI.HtmlControls;
    using System.Web.UI.WebControls;

    [ToolboxData("<{0}:AceEditor runat=\"server\" />")]
    public class AceEditor : Control
    {
        public AceEditor()
        {
            this.Controls.Add(new HiddenField { ID = "hiddenField" });
            this.Controls.Add(new HtmlGenericControl("div") { ID = "editDiv" });            
        }

        #region Public Properties

        [Category("MinCms")]
        public string DivStyleHeight { get; set; }

        [Category("MinCms")]
        public string DivStyleWidth { get; set; }

        [Category("MinCms")]
        [DefaultValue(null)]
        public string OffsetY { get; set; }

        [Category("MinCms")]
        [DefaultValue(null)]
        public string TextData
        {
            get
            {
                return this.Data.Value.Replace("@@", Environment.NewLine);
            }

            set
            {
                this.Data.Value = value.Replace("\r\n", "@@").Replace("\n", "@@");
            }
        }

        #endregion

        #region Properties

        private HiddenField Data
        {
            get { return this.Controls[0] as HiddenField; }
        }

        private HtmlGenericControl DivControl
        {
            get { return this.Controls[1] as HtmlGenericControl; }
        }

        [Category("MinCms")]
        private CssStyleCollection DivStyle
        {
            get { return this.DivControl.Attributes.CssStyle; }
        }

        private string Script
        {
            get
            {
                return "<script type=\"text/javascript\">$(function () {var editor = ace.edit(\"" + this.DivControl.ClientID +
                       "\");editor.getSession().setMode(\"ace/mode/html\");var hiddenField = $(\"#" + this.Data.ClientID +
                       "\"); editor.setValue(hiddenField.val()); editor.find('@@');editor.replaceAll('\\n');" +
                       "editor.getSession().on('change', function (e) { hiddenField.val(editor.getValue());});});</script>";
            }
        }

        #endregion

        #region Methods

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            this.DivControl.ID = this.ClientID;
            if (string.IsNullOrEmpty(this.DivStyleHeight))
            {
                this.DivStyle["position"] = "absolute";
                this.DivStyle["top"] = this.OffsetY == null ? "0" : this.OffsetY.ToString(CultureInfo.InvariantCulture) + "px";
                this.DivStyle["right"] = "0";
                this.DivStyle["bottom"] = "0";
                this.DivStyle["left"] = "0";
            }

            if (!string.IsNullOrEmpty(this.DivStyleWidth))
            {
                this.DivStyle["width"] = this.DivStyleWidth;
            }

            if (!string.IsNullOrEmpty(this.DivStyleHeight))
            {
                this.DivStyle["height"] = this.DivStyleHeight;
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            if (this.DesignMode)
            {
                writer.WriteLine("ACE-EDITOR");
            }
            else
            {
                this.RenderChildren(writer);
                writer.WriteLine(this.Script);
            }
        }

        #endregion
    }
}