﻿// MinCMS noninvasive ASP.NET WebForms System
// ----------------------------------------------------
// License: Microsoft Reciprocal License (Ms-RL)
// Version: 1.0.0.0
// Initial Development: Prosodev - Software Company / August 2013 / V 1.0.0.0

namespace MinCms.Core.Ui
{
    using System;
    using System.Linq;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    [ToolboxData("<{0}:MinCmsContentControl runat=\"server\" />")]
    public class MinCmsContentControl : Control, IMinCmsControlConfiguration, IMinCmsPreAspNetRendering, IPostBackEventHandler
    {
        #region Constructors and Destructors

        public MinCmsContentControl()
        {
            this.ValidateRequestMode = ValidateRequestMode.Disabled;
        }

        #endregion

        #region Public Properties

        public string Configuration { get; set; }

        #endregion

        #region Public Methods and Operators

        public string GetPreAspNetRendering(MinCms.IFile file)
        {
            if (!MinCms.Instance.EditMode)
            {
                return this.GetData(file);
            }

            return "!!LoadControl;" + MinCms.Instance.Plugins.First(a => a.GetAdminDisplayName() == "ContentControl").FullTypeName +
                   (string.IsNullOrEmpty(this.Configuration) || this.Configuration.StartsWith(";") ? this.Configuration : ";" + this.Configuration) + "!";
        }

        public void RaisePostBackEvent(string eventArgument)
        {
            if (eventArgument.StartsWith("edit"))
            {
                var hf = this.Controls[0] as HiddenField;
                if (hf != null)
                {
                    var val = hf.Value;
                    this.SetData(this.Page.Request.ToIFile(), val);
                }
            }
        }

        #endregion

        #region Methods

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (this.DesignMode)
            {
                return;
            }

            var hiddenField1 = new HiddenField {ID = "HiddenField" + this.ClientID};
            this.Controls.Add(hiddenField1);
        }

        protected override void Render(HtmlTextWriter writer)
        {
            writer.WriteLine("<div class=\"editable\">");
            writer.WriteLine(this.GetData(this.Page.Request.ToIFile()));
            writer.WriteLine("</div>");
            var pbo = new PostBackOptions(this) { Argument = "edit" }; 
            var script = this.Page.ClientScript.GetPostBackEventReference(pbo) + "; return false;";
            this.RenderChildren(writer);
            writer.WriteLine("<button style=\"display: none;\" onclick=\"{0}\"></button>", script);
        }

        private static void GetContentKeys(MinCms.IFile file, string configuration, out string fileKey, out string contentKey)
        {
            if (configuration != null && configuration.StartsWith(";"))
            {
                configuration = configuration.Substring(1);
            }

            if (string.IsNullOrEmpty(configuration))
            {
                fileKey = file.Url;
                contentKey = "Content";
            }
            else
            {
                var d = configuration.Split(';');
                fileKey = d.Length > 1 ? d[1] : file.Url;
                contentKey = d.Length > 1 ? d[0] : configuration;
            }
        }

        private string GetData(MinCms.IFile file)
        {
            string fileKey;
            string contentKey;
            GetContentKeys(file, this.Configuration, out fileKey, out contentKey);
            var val = MinCms.Instance.Database.GetValue(fileKey, contentKey, string.Empty);
            if (string.IsNullOrEmpty(val))
            {
                val = "<p>No content</p>";
            }
            return val;
        }

        private void SetData(MinCms.IFile file, string value)
        {
            string fileKey;
            string contentKey;
            GetContentKeys(file, this.Configuration, out fileKey, out contentKey);
            MinCms.Instance.Database.SetValue(fileKey, contentKey, value);
        }
        #endregion
    }
}