﻿// MinCMS noninvasive ASP.NET WebForms System
// ----------------------------------------------------
// License: Microsoft Reciprocal License (Ms-RL)
// Version: 1.0.0.0
// Initial Development: Prosodev - Software Company / August 2013 / V 1.0.0.0

namespace MinCms.Core.Ui
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    public partial class MinCmsFileEditor : UserControl
    {
        #region Methods

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.UpdateButtonDisplayValues();

            var db = MinCms.Instance.Properties["MinCmsFileEditor_Db", "True"] == "True";
            var file = MinCms.Instance.Properties["MinCmsFileEditor_File", "True"] == "True";
            var system = MinCms.Instance.Properties["MinCmsFileEditor_System", "False"] == "True";
            var addFileAsSystem = MinCms.Instance.Properties["MinCmsFileEditor_AddFilesAsSystem", "False"] == "True";
            var asp = MinCms.Instance.Properties["MinCmsFileEditor_Asp", "True"] == "True";
            var images = MinCms.Instance.Properties["MinCmsFileEditor_Images", "True"] == "True";
            var other = MinCms.Instance.Properties["MinCmsFileEditor_Other", "True"] == "True";
            var withTemplates = MinCms.Instance.Properties["MinCmsFileEditor_WithTemplates", "False"] == "True";

            this.Button10.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_Db"] = db ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };

            this.Button11.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_File"] = file ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };

            this.Button12.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_System"] = system ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };

            this.Button13.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_AddFilesAsSystem"] = addFileAsSystem ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };

            this.Button14.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_Asp"] = asp ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };

            this.Button15.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_Images"] = images ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };

            this.Button16.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_Other"] = other ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };

            this.Button17.ServerClick += (a, b) =>
            {
                MinCms.Instance.Properties["MinCmsFileEditor_WithTemplates"] = withTemplates ? "False" : "True";
                this.UpdateButtonDisplayValues();
                this.Repeater1.DataBind();
            };


            this.Repeater1.ItemCommand += (a, b) =>
            {
                var index = int.Parse(b.CommandArgument.ToString());
                var selfile = FileItem.Select()[index];
                switch (b.CommandName)
                {
                    case "upload":
                        var dir = Path.GetDirectoryName(selfile.Parent.PhysicalRelativePath);
                        if (!string.IsNullOrEmpty(dir))
                        {
                            foreach (var uplfile in MinCms.Instance.Files.Where(c => c.PhysicalRelativePath.StartsWith(dir)))
                            {
                                MinCms.Instance.Database.AddFile(uplfile, null, null, MinCms.Instance.Properties["MinCmsFileEditor_AddFilesAsSystem", "False"] == "True");
                            }
                        }
                        break;
                    case "uploadFile":
                        MinCms.Instance.Database.AddFile(selfile.Parent, null, null, MinCms.Instance.Properties["MinCmsFileEditor_AddFilesAsSystem", "False"] == "True");
                        break;
                    case "delete":
                        MinCms.Instance.Database.DeleteFile(selfile.Parent);
                        break;
                    case "editButton":
                        var pg = (string)HttpContext.Current.Session["PluginConfigurationCall_lastDest"];
                        HttpContext.Current.Session["PluginConfigurationCall"] = pg;
                        HttpContext.Current.Session["PluginConfigurationCall_plugin"] = MinCms.Instance.Plugins.First(x => x.GetAdminDisplayName() == "Edit template");
                        HttpContext.Current.Session["PluginConfigurationCall_file"] = selfile.Parent;
                        HttpContext.Current.Response.Redirect("~/default.aspx");
                        break;
                }

                this.Repeater1.DataBind();
            };

            this.Repeater1.ItemDataBound += (a, b) =>
            {
                var fi = b.Item.DataItem as FileItem;
                var uploadDir = b.Item.FindControl("uploadButton") as Button;
                var uploadFile = b.Item.FindControl("uploadFileButton") as Button;
                var deleteButton = b.Item.FindControl("deleteButton") as Button;
                var editButton = b.Item.FindControl("editButton") as Button;
                if (fi == null || uploadDir == null || uploadFile == null || deleteButton == null || editButton == null)
                {
                    return;
                }

                uploadDir.CommandName = "upload";
                uploadDir.CommandArgument = fi.Index.ToString(CultureInfo.InvariantCulture);
                uploadFile.CommandName = "uploadFile";
                uploadFile.CommandArgument = fi.Index.ToString(CultureInfo.InvariantCulture);
                deleteButton.CommandName = "delete";
                deleteButton.CommandArgument = fi.Index.ToString(CultureInfo.InvariantCulture);
                editButton.CommandName = "editButton";
                editButton.CommandArgument = fi.Index.ToString(CultureInfo.InvariantCulture);
                if (!fi.Parent.Url.EndsWith("ascx") && !fi.Parent.Url.EndsWith("aspx"))
                {
                    editButton.Enabled = false;
                }

                if (!fi.Parent.ExistsInDatabase)
                {
                    deleteButton.Enabled = false;
                    editButton.Enabled = false;
                }
            };

            this.Repeater1.DataBind();
        }

        private void UpdateButtonDisplayValues()
        {
            var db = MinCms.Instance.Properties["MinCmsFileEditor_Db", "True"] == "True";
            var file = MinCms.Instance.Properties["MinCmsFileEditor_File", "True"] == "True";
            var system = MinCms.Instance.Properties["MinCmsFileEditor_System", "False"] == "True";
            var addFileAsSystem = MinCms.Instance.Properties["MinCmsFileEditor_AddFilesAsSystem", "False"] == "True";
            var asp = MinCms.Instance.Properties["MinCmsFileEditor_Asp", "True"] == "True";
            var images = MinCms.Instance.Properties["MinCmsFileEditor_Images", "True"] == "True";
            var other = MinCms.Instance.Properties["MinCmsFileEditor_Other", "True"] == "True";
            var withTemplates = MinCms.Instance.Properties["MinCmsFileEditor_WithTemplates", "False"] == "True";

            this.Button10.InnerText = "Db (" + (db ? "on" : "off") + ")";
            this.Button11.InnerText = "File (" + (file ? "on" : "off") + ")";
            this.Button12.InnerText = "System (" + (system ? "on" : "off") + ")";
            this.Button13.InnerText = "Add as system file (" + (addFileAsSystem ? "on" : "off") + ")";
            this.Button14.InnerText = "Asp.NET (" + (asp ? "on" : "off") + ")";
            this.Button15.InnerText = "Images (" + (images ? "on" : "off") + ")";
            this.Button16.InnerText = "Other (" + (other ? "on" : "off") + ")";
            this.Button17.InnerText = "With templates (" + (withTemplates ? "on" : "off") + ")";
        }

        protected void OnCloseWithoutSaving(object sender, EventArgs e)
        {
            MinCms.Instance.ClearMinCmsPluginConfiguration();
        }

        #endregion
    }

    public sealed class FileItem
    {
        #region Fields

        public MinCms.IFile Parent { get; private set; }

        #endregion

        #region Constructors and Destructors

        private FileItem(MinCms.IFile parent)
        {
            this.Parent = parent;
            var file = System.IO.Path.GetFileName(this.Parent.PhysicalPath);
            if (!string.IsNullOrEmpty(file))
            {
                this.Path = this.Parent.PhysicalRelativePath.Substring(0, this.Parent.PhysicalRelativePath.Length - file.Length);
                this.FileName = file;
            }
        }

        #endregion

        #region Public Properties

        public string ExistsOnDatabase
        {
            get { return this.Parent.ExistsInDatabase ? "<span class=\"glyphicon glyphicon-ok\"></span>" : string.Empty; }
        }

        public string ExistsOnDisc
        {
            get { return this.Parent.ExistsOnDisc ? "<span class=\"glyphicon glyphicon-ok\"></span>" : string.Empty; }
        }

        public string FileName { get; private set; }

        public string Path { get; private set; }

        public string SystemFile
        {
            get { return this.Parent.SystemFile ? "<span class=\"glyphicon glyphicon-ok\"></span>" : string.Empty; }
        }

        #endregion

        #region Public Methods and Operators

        public static List<FileItem> Select()
        {
            var db = MinCms.Instance.Properties["MinCmsFileEditor_Db", "True"] == "True";
            var file = MinCms.Instance.Properties["MinCmsFileEditor_File", "True"] == "True";
            var system = MinCms.Instance.Properties["MinCmsFileEditor_System", "False"] == "True";
            var asp = MinCms.Instance.Properties["MinCmsFileEditor_Asp", "True"] == "True";
            var images = MinCms.Instance.Properties["MinCmsFileEditor_Images", "True"] == "True";
            var other = MinCms.Instance.Properties["MinCmsFileEditor_Other", "True"] == "True";
            var withTemplates = MinCms.Instance.Properties["MinCmsFileEditor_WithTemplates", "False"] == "True";

            var filesToReturn = MinCms.Instance.Files.Where(
                a => ((a.Url.EndsWith("aspx") || a.Url.EndsWith("ascx")) && asp) ||
                     ((a.Url.EndsWith("gif") || a.Url.EndsWith("jpg") || a.Url.EndsWith("jpeg") || a.Url.EndsWith("png") || a.Url.EndsWith("ico") || a.Url.EndsWith("svg")) && images)
                     || other
                )
                .Where(a => !withTemplates || a.Template != null)
                .Where(a => !a.ExistsInDatabase && !db || db)
                .Where(a => !a.ExistsOnDisc && !file || file)
                .Where(a => !a.SystemFile && !system || system)
                .Select(a => new FileItem(a))
                .ToList();
            for (int i = 0; i < filesToReturn.Count; i++)
            {
                filesToReturn[i].Index = i;
            }
            return filesToReturn;
        }

        public int Index { get; private set; }

        #endregion
    }
}