﻿// MinCMS noninvasive ASP.NET WebForms System
// ----------------------------------------------------
// License: Microsoft Reciprocal License (Ms-RL)
// Version: 1.0.0.0
// Initial Development: Prosodev - Software Company / August 2013 / V 1.0.0.0

namespace MinCms
{
    using System;
    using System.Globalization;
    using System.Linq;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.Threading;
    using System.Web.Services;
    using System.Web.UI;
    using System.Web.UI.WebControls;
    using Core;
    using Service;

    public partial class Default : Page
    {
        #region Static Fields

        private static readonly object LockObject = new object();

        private static int upgradePercentageProp;

        #endregion

        #region Properties

        private static bool UpgradeInProgressProp { get; set; }

        private static string UpgradeMessageProp { get; set; }

        private static int UpgradePercentageProp
        {
            get { return upgradePercentageProp; }
            set
            {
                upgradePercentageProp = value <= 100 ? value : 0;

                if (value < 0)
                {
                    upgradePercentageProp = 0;
                }
            }
        }

        #endregion

        #region Public Methods and Operators

        [WebMethod]
        public static bool UpgradeInProgress()
        {
            if (!MinCms.Instance.LocalhostMode)
            {
                return false;
            }

            lock (LockObject)
            {
                return UpgradeInProgressProp;
            }
        }

        [WebMethod]
        public static string UpgradeMessage()
        {
            if (!MinCms.Instance.LocalhostMode)
            {
                return null;
            }

            lock (LockObject)
            {
                return UpgradeMessageProp + " " + UpgradePercentageProp + "%";
            }
        }

        [WebMethod]
        public static string UpgradePercentage()
        {
            if (!MinCms.Instance.LocalhostMode)
            {
                return "0";
            }

            lock (LockObject)
            {
                return UpgradePercentageProp.ToString(CultureInfo.InvariantCulture);
            }
        }

        [WebMethod]
        public static string UpgradeStart()
        {
            if (!MinCms.Instance.LocalhostMode)
            {
                return string.Empty;
            }

            UpgradeMessageProp = "Started...";
            UpgradeInProgressProp = true;
            UpgradePercentageProp = 10;

            var th = new Thread(UpgradeRemote);
            th.Start();

            lock (LockObject)
            {
                return string.Empty;
            }
        }

        #endregion

        #region Methods

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (this.Session["PluginConfigurationCall"] != null)
            {
                var plg = (IMinCmsPlugin)this.Session["PluginConfigurationCall_plugin"];
                var pg = (string)this.Session["PluginConfigurationCall"];
                var il = (MinCms.IFile)this.Session["PluginConfigurationCall_file"];
                this.Session["PluginConfigurationCall_lastDest"] = pg;
                this.Header.Controls.Add(new Literal { Text = plg.GetAdminPageIncludes() });
                this.Form.Controls.Clear();
                this.Form.Controls.Add(plg.GetAdminControl(this, il));
                return;
            }

            if (this.Session["CallBackCurrentPage"] != null)
            {
                var callbackUrl = this.Session["CallBackCurrentPage"] as string;
                if (!string.IsNullOrEmpty(callbackUrl))
                {
                    this.Session["CallBackCurrentPage"] = null;
                    var file = MinCms.Instance.Files.FirstOrDefault(a => a.Url == callbackUrl.ToLower());
                    if (file != null)
                    {
                        var func = "setTimeout(function() {window.location.href = \"" + this.ResolveUrl(file.Url) + "\";}, 2000);";
                        this.Literal1.Text = ("<script type=\"text/javascript\">" + func + "</script>") + "<h1>You are redirected...</h1>";
                        return;
                    }
                }
            }

            if (MinCms.Instance.DefaultPage != null)
            {
                this.Response.Redirect(MinCms.Instance.DefaultPage.Url);
            }
        }

        private static void UpgradeRemote()
        {
            var binding = MinCmsHelper.RemoteSystemHttps ? (Binding)new BasicHttpsBinding() : new BasicHttpBinding();
            var adr = new EndpointAddress(MinCmsHelper.RemoteSystem);
            var client = new DatabaseExchangeSoapClient(binding, adr);
            var dt = client.GetLastUpdate(MinCmsHelper.AccessToken);
            var count = MinCms.Instance.Database.GetMinCmsFileUpdateListCounter(dt);
            var i = 0;
            foreach (var f in MinCms.Instance.Database.GetMinCmsFileUpdateList(dt))
            {
                lock (LockObject)
                {
                    UpgradePercentageProp = count/100 == 0 ? 1 : i/(count/100);
                    UpgradeMessageProp = "Uploading files";
                }

                client.UpdateMinCmsFile(MinCmsHelper.AccessToken, f);
                i += 1;
            }
            
            count = MinCms.Instance.Database.GetMinCmsPropertyUpdateListCounter(dt);
            i = 0;
            foreach (var f in MinCms.Instance.Database.GetMinCmsPropertyUpdateList(dt))
            {
                lock (LockObject)
                {
                    UpgradePercentageProp = count/100 == 0 ? 1 : i/(count/100);
                    UpgradeMessageProp = "Uploading properties";
                }

                client.UpdateMinCmsProperty(MinCmsHelper.AccessToken, f);
                i += 1;
            }



            lock (LockObject)
            {
                UpgradePercentageProp = 100;
                UpgradeMessageProp = "Finished";
                UpgradeInProgressProp = false;
            }
        }

        #endregion
    }
}